<?php

namespace BexioConnectorByHelvy;

/**
 * Handle WooCommerce Webhooks Events
 *
 * @since 0.1.0
 */

defined('ABSPATH') || exit;

class WooCommerceWebhooks
{

	public function __construct()
	{
        add_action('woocommerce_new_order', array($this, 'handle_order_created'), 10, 2);
        add_action('woocommerce_order_status_processing', array($this, 'handle_order_paid'));
        add_action('woocommerce_order_status_completed', array($this, 'handle_order_completed'));
        add_filter('woocommerce_email_attachments', array($this, 'attach_invoice_pdf_to_wc_emails'), 10, 3 );
	}

    /**
     * Triggered when a new order is created
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_created($order_id, $order) {
        // Skip processing if the order is in a non-syncable state
        $order_status = $order->get_status();
        if (in_array($order_status, array('draft', 'cancelled', 'refunded', 'failed'))) {
            return;
        }

        WCToBexio::instance()->sync_order($order);
        WCToBexio::instance()->sync_order_payment($order);
        WCToBexio::instance()->sync_order_fulfillment($order);
    }

    /**
     * Triggered when an order is paid
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_paid($order_id) {
        $order = wc_get_order($order_id);
        WCToBexio::instance()->sync_order_payment($order);
    }

    /**
     * Triggered when an order is completed
     * @param $order_id int Order ID
     * @return void
     */
	function handle_order_completed($order_id) {
        $order = wc_get_order($order_id);
        WCToBexio::instance()->sync_order_fulfillment($order);
    }

    /**
     * Attach the Bexio invoice PDF to WooCommerce emails
     * @param $attachments array Array of attachments
     * @param $email_id string Email ID
     * @param $order WC_Order Order object
     * @return array Array of attachments
     */
    function attach_invoice_pdf_to_wc_emails( $attachments, $email_id, $order ) {
        if (!$order instanceof \WC_Order) {
            return $attachments;
        }
        
        if (!get_option(AdminAPISettings::$prefix . 'attach_bexio_invoice_to_woocommerce_email')) {
            return $attachments;
        }
        
        // Check if this is the 'customer_processing_order' or 'customer_on_hold_order' email (email sent right after the order is placed)
        if ($email_id === 'customer_processing_order' || $email_id === 'customer_on_hold_order') {
            // Check for the invoice ID with a maximum of 10 attempts, one per second
            $max_attempts = 10;
            $attempts = 0;
            $invoice_id = '';
            while ($attempts < $max_attempts) {
                $invoice_id = $order->get_meta(AdminAPISettings::$prefix . 'bexio_invoice_id');
                if ($invoice_id) {
                    break;
                }
                $attempts++;
                sleep(1);
            }

            // Invoice not available
            if (!$invoice_id) {
                error_log("No invoice ID found after $max_attempts attempts for order " . $order->get_id());
                return $attachments;
            }

            $invoice_pdf = API::instance()->get_invoice_pdf($invoice_id);
            if ($invoice_pdf instanceof \WP_Error) {
                error_log("Not able to retrieve invoice PDF for order " . $order->get_id());
                return $attachments;
            }

            // Fetch the invoice in base64 format from an API
            $invoice_base64 = $invoice_pdf["content"];
            if ( $invoice_base64 ) {
                $decoded_pdf = base64_decode( $invoice_base64 );
                if ( $decoded_pdf === false ) {
                    error_log( 'Failed to decode base64 PDF invoice for order ID ' . $order->get_id() );
                    return $attachments;
                }

                // Upload invoice to the uploads directory
                $upload_dir = wp_upload_dir();
                $upload_path = $upload_dir['basedir'] . '/invoices/';
                if ( ! file_exists( $upload_path ) ) {
                    mkdir( $upload_path, 0755, true );
                }

                $secure_id = bin2hex(random_bytes(16)); // Generates a 32-character hexadecimal string
                $file_path = $upload_path . 'invoice-' . $secure_id . '-' . $invoice_pdf["name"];
                $file_written = file_put_contents( $file_path, $decoded_pdf );
                if ( $file_written === false ) {
                    error_log( 'Failed to write the decoded PDF to the file for order ID ' . $order->get_id() );
                    return $attachments;
                }

                // Attach the file to the email
                if ( file_exists( $file_path ) ) {
                    $attachments[] = $file_path;
                }
            }
        }
    
        return $attachments;
    }
    
}
